#include "stdafx.h"
#include <Windows.h>
#include <cstdio>

#pragma comment(lib, "User32.lib")

static CSCPController* l_Pad[XUSER_MAX_COUNT];
static BOOL  l_bPassThrough = false, l_bStarted = false;
static DWORD l_nPads = 0, l_nAttached = 0;

BOOL LoadApi(BOOL bAttach)
{
	if (bAttach) l_nAttached++; else l_nAttached--;

	if (bAttach && !l_bStarted)
	{
		load_lib_usb();
		WRAP_LoadXInput(true);

		CSCPController*	Pad;

		// Load DS3s
		for (DWORD Index = 0; Index < XUSER_MAX_COUNT * CDS3Controller::CollectionSize && l_nPads < XUSER_MAX_COUNT; Index++)
		{
			Pad = new CDS3Controller(Index);

			if (Pad->Open()) l_Pad[l_nPads++] = Pad;
			else { delete Pad; break; }
		}

		// Load SL3s
		for (DWORD Index = 0; Index < XUSER_MAX_COUNT * CSL3Controller::CollectionSize && l_nPads < XUSER_MAX_COUNT; Index++)
		{
			Pad = new CSL3Controller(Index);

			if (Pad->Open()) l_Pad[l_nPads++] = Pad;
			else { delete Pad; break; }
		}

		// Load DS2s
		for (DWORD Index = 0; Index < XUSER_MAX_COUNT * CDS2Controller::CollectionSize && l_nPads < XUSER_MAX_COUNT; Index++)
		{
			Pad = new CDS2Controller(Index);

			if (Pad->Open()) l_Pad[l_nPads++] = Pad;
			else delete Pad;
		}

		l_bStarted = true;
	}
	else if (!bAttach && l_bStarted && (l_nAttached == 0))
	{
		l_bStarted = false;

		for (DWORD Index = 0; Index < l_nPads; Index++)
		{
			l_Pad[Index]->Close();
			delete l_Pad[Index];
		}

		return WRAP_LoadXInput(false);
	}

	return true;
}

DWORD WINAPI XInputGetState
(
    __in  DWORD         dwUserIndex,						// Index of the gamer associated with the device
    __out XINPUT_STATE* pState								// Receives the current state
)
{
	if (dwUserIndex < l_nPads)
	{
		return l_Pad[dwUserIndex]->GetState(dwUserIndex, pState);
	}
	else
	{
		return WRAP_XInputGetState(dwUserIndex - l_nPads, pState);
	}
}

DWORD WINAPI XInputSetState
(
    __in DWORD             dwUserIndex,						// Index of the gamer associated with the device
    __in XINPUT_VIBRATION* pVibration						// The vibration information to send to the controller
)
{
	if (dwUserIndex < l_nPads)
	{
		return l_Pad[dwUserIndex]->SetState(dwUserIndex, pVibration);
	}
	else
	{
		return WRAP_XInputSetState(dwUserIndex - l_nPads, pVibration);
	}
}

DWORD WINAPI XInputGetCapabilities
(
    __in  DWORD                dwUserIndex,					// Index of the gamer associated with the device
    __in  DWORD                dwFlags,						// Input flags that identify the device type
    __out XINPUT_CAPABILITIES* pCapabilities				// Receives the capabilities
)
{
	if (dwUserIndex < l_nPads)
	{
		return l_Pad[dwUserIndex]->GetCapabilities(dwUserIndex, dwFlags, pCapabilities);
	}
	else
	{
		return WRAP_XInputGetCapabilities(dwUserIndex - l_nPads, dwFlags, pCapabilities);
	}
}

void WINAPI XInputEnable
(
    __in BOOL enable										// [in] Indicates whether xinput is enabled or disabled. 
)
{
	// Pause/resume DS3 rumble here?
	WRAP_XInputEnable(enable);
}

DWORD WINAPI XInputGetDSoundAudioDeviceGuids
(
    __in  DWORD dwUserIndex,								// Index of the gamer associated with the device
    __out GUID* pDSoundRenderGuid,							// DSound device ID for render
    __out GUID* pDSoundCaptureGuid							// DSound device ID for capture
)
{
	if (dwUserIndex < l_nPads)
	{
		return l_Pad[dwUserIndex]->GetDSoundAudioDeviceGuids(dwUserIndex, pDSoundRenderGuid, pDSoundCaptureGuid);
	}
	else
	{
		return WRAP_XInputGetDSoundAudioDeviceGuids(dwUserIndex - l_nPads, pDSoundRenderGuid, pDSoundCaptureGuid);
	}
}

DWORD WINAPI XInputGetBatteryInformation
(
    __in  DWORD                       dwUserIndex,			// Index of the gamer associated with the device
    __in  BYTE                        devType,				// Which device on this user index
    __out XINPUT_BATTERY_INFORMATION* pBatteryInformation	// Contains the level and types of batteries
)
{
	if (dwUserIndex < l_nPads)
	{
		return l_Pad[dwUserIndex]->GetBatteryInformation(dwUserIndex, devType, pBatteryInformation);
	}
	else
	{
		return WRAP_XInputGetBatteryInformation(dwUserIndex - l_nPads, devType, pBatteryInformation);
	}
}

DWORD WINAPI XInputGetKeystroke
(
    __in       DWORD dwUserIndex,							// Index of the gamer associated with the device
    __reserved DWORD dwReserved,							// Reserved for future use
    __out      PXINPUT_KEYSTROKE pKeystroke					// Pointer to an XINPUT_KEYSTROKE structure that receives an input event.
)
{
	if (dwUserIndex < l_nPads)
	{
		return l_Pad[dwUserIndex]->GetKeystroke(dwUserIndex, dwReserved, pKeystroke);
	}
	else
	{
		return WRAP_XInputGetKeystroke(dwUserIndex - l_nPads, dwReserved, pKeystroke);
	}
}

DWORD WINAPI XInputGetExtended(DWORD dwUserIndex, SCP_EXTN *Pressure)
{
	if (dwUserIndex < l_nPads)
	{
		return l_Pad[dwUserIndex]->GetExtended(dwUserIndex, Pressure);
	}
	else
	{
		return ERROR_DEVICE_NOT_CONNECTED; // Passthrough it!
	}
}