#include "StdAfx.h"
#define XINPUT_FUNCTIONS 7

static HMODULE l_hXInputDll = NULL;
static FARPROC l_hXInputFunc[] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL }; 

typedef DWORD (WINAPI *XInputGetStateFunction)(__in DWORD dwUserIndex, __out XINPUT_STATE* pState);
typedef DWORD (WINAPI *XInputSetStateFunction)(__in DWORD dwUserIndex, __in  XINPUT_VIBRATION* pVibration);
typedef DWORD (WINAPI *XInputGetCapabilitiesFunction)(__in DWORD dwUserIndex, __in DWORD dwFlags, __out XINPUT_CAPABILITIES* pCapabilities);
typedef void  (WINAPI *XInputEnableFunction)(__in BOOL enable);
typedef DWORD (WINAPI *XInputGetDSoundAudioDeviceGuidsFunction)(__in DWORD dwUserIndex, __out GUID* pDSoundRenderGuid, __out GUID* pDSoundCaptureGuid);
typedef DWORD (WINAPI *XInputGetBatteryInformationFunction)(__in DWORD dwUserIndex, __in BYTE devType, __out XINPUT_BATTERY_INFORMATION* pBatteryInformation);
typedef DWORD (WINAPI *XInputGetKeystrokeFunction)(__in DWORD dwUserIndex, __reserved DWORD dwReserved, __out PXINPUT_KEYSTROKE pKeystroke);

enum
{
	XGETSTATE,
	XSETSTATE,
	XGETCAPS,
	XENABLE,
	XGETDSOUND,
	XGETBATTERY,
	XGETKEY
};

BOOL WINAPI WRAP_LoadXInput
(
	__in BOOL enable
)
{
static BOOL bInited = false;

	if (enable && !bInited)
	{
		TCHAR libdir[MAX_PATH];
		GetSystemDirectory(libdir, MAX_PATH);
		
#ifndef XINPUT_USE_9_1_0
		_stprintf_s(libdir, _T("%s\\XInput1_3.dll"), libdir); 
#else
		_stprintf_s(libdir, _T("%s\\XInput9_1_0.dll"), libdir);
#endif //!XINPUT_USE_9_1_0
		if ((l_hXInputDll = LoadLibrary(libdir)) != NULL)
		{
			l_hXInputFunc[XGETSTATE]	= GetProcAddress(l_hXInputDll, "XInputGetState");
			l_hXInputFunc[XSETSTATE]	= GetProcAddress(l_hXInputDll, "XInputSetState");
			l_hXInputFunc[XGETCAPS]		= GetProcAddress(l_hXInputDll, "XInputGetCapabilities");
			l_hXInputFunc[XENABLE]		= GetProcAddress(l_hXInputDll, "XInputEnable");
			l_hXInputFunc[XGETDSOUND]	= GetProcAddress(l_hXInputDll, "XInputGetDSoundAudioDeviceGuids");
#ifndef XINPUT_USE_9_1_0
			l_hXInputFunc[XGETBATTERY]	= GetProcAddress(l_hXInputDll, "XInputGetBatteryInformation");
			l_hXInputFunc[XGETKEY]		= GetProcAddress(l_hXInputDll, "XInputGetKeystroke");
#endif //!XINPUT_USE_9_1_0

			bInited = true;
		}
	}
	else if (!enable && bInited)
	{
		for (int i = 0; i < XINPUT_FUNCTIONS; i++) l_hXInputFunc[i] = NULL;
		if (l_hXInputDll) FreeLibrary(l_hXInputDll); l_hXInputDll = NULL;
		bInited = false;
	}

	return true;
}

DWORD WINAPI WRAP_XInputGetState
(
    __in  DWORD         dwUserIndex,						// Index of the gamer associated with the device
    __out XINPUT_STATE* pState								// Receives the current state
)
{
	if (!l_hXInputFunc[XGETSTATE]) return ERROR_DEVICE_NOT_CONNECTED;

	return ((XInputGetStateFunction)(l_hXInputFunc[XGETSTATE]))(dwUserIndex, pState);
}

DWORD WINAPI WRAP_XInputSetState
(
    __in DWORD             dwUserIndex,						// Index of the gamer associated with the device
    __in XINPUT_VIBRATION* pVibration						// The vibration information to send to the controller
)
{
	if (!l_hXInputFunc[XSETSTATE]) return ERROR_DEVICE_NOT_CONNECTED;

	return ((XInputSetStateFunction)(l_hXInputFunc[XSETSTATE]))(dwUserIndex, pVibration);
}

DWORD WINAPI WRAP_XInputGetCapabilities
(
    __in  DWORD                dwUserIndex,					// Index of the gamer associated with the device
    __in  DWORD                dwFlags,						// Input flags that identify the device type
    __out XINPUT_CAPABILITIES* pCapabilities				// Receives the capabilities
)
{
	if (!l_hXInputFunc[XGETCAPS]) return ERROR_DEVICE_NOT_CONNECTED;

	return ((XInputGetCapabilitiesFunction)(l_hXInputFunc[XGETCAPS]))(dwUserIndex, dwFlags, pCapabilities);
}

void WINAPI WRAP_XInputEnable
(
    __in BOOL enable										// [in] Indicates whether xinput is enabled or disabled. 
)
{
	if (!l_hXInputFunc[XENABLE]) return;

	return ((XInputEnableFunction)(l_hXInputFunc[XENABLE]))(enable);
}

DWORD WINAPI WRAP_XInputGetDSoundAudioDeviceGuids
(
    __in  DWORD dwUserIndex,								// Index of the gamer associated with the device
    __out GUID* pDSoundRenderGuid,							// DSound device ID for render
    __out GUID* pDSoundCaptureGuid							// DSound device ID for capture
)
{
	if (!l_hXInputFunc[XGETDSOUND]) return ERROR_DEVICE_NOT_CONNECTED;

	return ((XInputGetDSoundAudioDeviceGuidsFunction)(l_hXInputFunc[XGETDSOUND]))(dwUserIndex, pDSoundRenderGuid, pDSoundCaptureGuid);
}

#ifndef XINPUT_USE_9_1_0

DWORD WINAPI WRAP_XInputGetBatteryInformation
(
    __in  DWORD                       dwUserIndex,			// Index of the gamer associated with the device
    __in  BYTE                        devType,				// Which device on this user index
    __out XINPUT_BATTERY_INFORMATION* pBatteryInformation	// Contains the level and types of batteries
)
{
	if (!l_hXInputFunc[XGETBATTERY]) return ERROR_DEVICE_NOT_CONNECTED;

	return ((XInputGetBatteryInformationFunction)(l_hXInputFunc[XGETBATTERY]))(dwUserIndex, devType, pBatteryInformation);
}

DWORD WINAPI WRAP_XInputGetKeystroke
(
    __in       DWORD dwUserIndex,							// Index of the gamer associated with the device
    __reserved DWORD dwReserved,							// Reserved for future use
    __out      PXINPUT_KEYSTROKE pKeystroke					// Pointer to an XINPUT_KEYSTROKE structure that receives an input event.
)
{
	if (!l_hXInputFunc[XGETKEY]) return ERROR_DEVICE_NOT_CONNECTED;

	return ((XInputGetKeystrokeFunction)(l_hXInputFunc[XGETKEY]))(dwUserIndex, dwReserved, pKeystroke);
}

#endif //!XINPUT_USE_9_1_0
